#!/usr/bin/python3
# -*- coding: utf-8 -*-
import os
import signal
import datetime
import logging
import dbus
import dbus.mainloop.glib
import dbus.service

from gi.repository import GLib
from threading import Thread

from update_thread import UpdateThread
from checkupdate import *

INTERFACE = 'com.nfs.update'
PATH = '/'

class DbusDaemon(dbus.service.Object):
    """
    """
    def __init__(self,mainloop=None):
        self.mainloop = mainloop
        bus_loop = dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
        bus = dbus.SystemBus(mainloop=bus_loop)
        bus_name = dbus.service.BusName(INTERFACE,bus)
        dbus.service.Object.__init__(self,bus_name,PATH)
        signal.signal(signal.SIGINT,lambda: mainloop.quit())
        self.updating = False
        self.update_thread = None
        self.new_version = (0,'')
        self.updates_list = []
        self.is_automatic = False
        self.new_version_thread = None
        self.updates_list_thread = None
        self.is_automatic_thread = None

    def thread_finished(self,update_thread,result):
        thread_data = update_thread.data
        name = update_thread.name
        changed = False
        if name == 'UPDATE_LIST':
            if self.updates_list != thread_data:
                self.updates_list = thread_data
                changed = True
        elif name == 'NEW_VERSION':
            if self.new_version != thread_data:
                self.new_version = thread_data
                changed = True
        elif name == 'IS_AUTOMATIC':
            if self.is_automatic != thread_data:
                self.is_automatic = thread_data
                changed = True    
        self.Finished(name,changed,'')

    def create_thread(self,name):
        thread = None
        try:
            thread = UpdateThread(name)
            thread.connect("finished",self.thread_finished)
            thread.start()
        except:
            logging.error("Create thread %s failed." % name)
            pass
        return thread

    @dbus.service.method(INTERFACE,in_signature='',out_signature='')
    def checkUpdate(self):
        '''
        threads: new_versionupdates_list
        '''
        if self.new_version_thread:
            self.new_version_thread.join()
        self.new_version_thread = self.create_thread('NEW_VERSION')

        if self.updates_list_thread:
            self.updates_list_thread.join()
        self.updates_list_thread = self.create_thread('UPDATE_LIST')

        if self.is_automatic_thread:
            self.is_automatic_thread.join()
        self.is_automatic_thread = self.create_thread('IS_AUTOMATIC')

    @dbus.service.method(INTERFACE,in_signature='',out_signature='(is)')
    def getNewVersion(self):
        return self.new_version

    def update_finished(self,update_thread,result):
        self.updating = False
        name = update_thread.name
        self.Finished(name,result,'')

    @dbus.service.method(INTERFACE,in_signature='',out_signature='b')
    def updateNow(self):
        ret = True
        if self.update_thread:
            self.update_thread.join()
            self.update_thread = None
        try:
            self.updating = True
            self.Finished("IS_UPDATING",True,'')
            self.update_thread = UpdateThread('UPDATE')
            self.update_thread.connect("finished",self.update_finished)
            self.update_thread.start()
        except:
            ret = False
        return ret

    @dbus.service.signal(dbus_interface=INTERFACE,signature='sbs')
    def Finished(self,action,result,msg=''):
        #msg is something aboubt error.
        pass

    @dbus.service.method(INTERFACE,in_signature='',out_signature='')
    def Exit(self):
        self.mainloop.quit()

    @dbus.service.method(INTERFACE,in_signature='',out_signature='b')
    def isUpdating(self):
        return self.updating

    @dbus.service.method(INTERFACE,in_signature='',out_signature='b')
    def isAutomatic(self):
        result = self.is_automatic
        return result

    @dbus.service.method(INTERFACE,in_signature='b',out_signature='b')
    def setAutomatic(self,auto):
        result = set_automatic(auto)
        if result:
            self.is_automatic = auto
            self.Finished('IS_AUTOMATIC',result,'')
        return result

    @dbus.service.method(INTERFACE,in_signature='',out_signature='aa{ss}')
    def getUpdatePkgs(self):
        updates = self.updates_list
        #if not updates:
        #updates = [{'name':'mytest''size': '1.7M'},{'name':'mytest1''size': '1.8M'},{'name':'mytest2''size': '1.9M'}]
        return updates

if __name__ == '__main__':
    loop = GLib.MainLoop()
    DbusDaemon(loop)
    loop.run()
