#/bin/python3

import os
import sys
import argparse

import dnf
import dnf.automatic.emitter
import dnf.cli
import dnf.cli.cli
import dnf.cli.output
import dnf.conf
import libdnf.conf
import dnf.const
import dnf.exceptions
import dnf.util
import pycurl
from io import BytesIO

here = sys.path[0]
if here != '/usr/bin':
    # git checkout
    dnf_toplevel = os.path.dirname(here)
    sys.path[0] = dnf_toplevel

import dnf.automatic.main

def parse_arguments(args):
    parser = argparse.ArgumentParser()
    parser.add_argument('conf_path', nargs='?', default='')
    parser.add_argument('--timer', action='store_true')
    parser.add_argument('--installupdates', dest='installupdates', action='store_true')
    parser.add_argument('--downloadupdates', dest='downloadupdates', action='store_true')
    parser.add_argument('--no-installupdates', dest='installupdates', action='store_false')
    parser.add_argument('--no-downloadupdates', dest='downloadupdates', action='store_false')
    parser.add_argument('--updateinfo', dest='updateinfo', action='store_true')
    parser.add_argument('--isautomatic', dest='isautomatic', action='store_true')
    parser.set_defaults(installupdates=None)
    parser.set_defaults(downloadupdates=None)

    return parser.parse_args(args), parser

(opts, parser) = parse_arguments(sys.argv[1:])

def update():
    pfd = os.popen('/usr/bin/dnf-automatic --installupdates --timer')
    lines = pfd.read()
    for line in lines.splitlines():
        print(line)
    return pfd.close()
    #sys.exit(dnf.automatic.main.main(['/etc/dnf/automatic.conf', '--installupdates', '--timer']))

def get_os_version_http(url):
    result = ""
    ##url = 'http://172.30.13.15/repos/NFS4.0/GenOS/RPMS/latestversion/nfs-release'
    crl = pycurl.Curl() #创建一个pycurl对象的方法
    crl.fp = BytesIO() #设置IO，py3要求使用对bytes操作的BytesIO
    crl.setopt(crl.URL, url) #设置请求的Url
    crl.setopt(pycurl.TIMEOUT, 5)   #请求超时 
    crl.setopt(crl.WRITEFUNCTION, crl.fp.write) #将返回的内容定向到回调函数crl.fp.write，作用是将内容写入crl.fp
    try:
        crl.perform()
        code = crl.getinfo(pycurl.HTTP_CODE)
        if code == 200:
            version = crl.fp.getvalue()
            result = str(version.decode('utf-8').replace('\n', ''))
    except:
        code = -1
    return (code, result)

def readfile(path):
    try:
        with open(path, 'r') as fp:
            ret = fp.read()
            fp.close()
            return ret.strip()
    except:
        return ""

def get_os_version_ftp(url):
    result = ""
    ##url = 'ftp://updates.os.nfschina.com/NFS4.0/ArmOS/RPMS/aarch64/latestversion/nfs-release'
    ver = "/tmp/.osver"
    cmd = "wget -q " + url + " -O " + ver
    try:
        os.system(cmd)
        if os.path.exists(ver):
            result = readfile(ver)
            if result == "" :
                code = -1
            else:
                code = 200
    except:
        code = -1
    return (code, result)

#arm version repo is ftp.
def get_os_version_default():
    url = 'ftp://updates.os.nfschina.com/NFS4.0/ArmOS/RPMS/aarch64/latestversion/nfs-release'
    return get_os_version_ftp(url)

def get_os_version():
    ret = get_os_version_default()
    #Can not found.
    if ret[0] != 200:
        #read status by conf.
        conf = "/opt/nfs-upgrade-tool/new_version_conf"
        if os.path.exists(conf):
            conf = readfile(conf)
            #print (conf)
            tmp = conf.split("://")
            #print (tmp[0].strip())
            if tmp[0].strip() == "http":
                ret = get_os_version_http(conf)
            elif tmp[0].strip() == "ftp":
                ret = get_os_version_ftp(conf)
    #print (ret)
    return ret

def disable_automatic():
    os.popen('systemctl stop dnf-automatic-install.timer')
    pfd = os.popen('systemctl disable dnf-automatic-install.timer')
    lines = pfd.read()
    for line in lines.splitlines():
        print(line)
    res = pfd.close()
    # pfd = os.popen('systemctl stop dnf-automatic.timer')
    # lines = pfd.read()
    # for line in lines.splitlines():
    #     print(line)
    # res = res and pfd.close()
    
    return res

def enable_automatic():
    os.popen('systemctl start dnf-automatic-install.timer')
    pfd = os.popen('systemctl enable dnf-automatic-install.timer')
    lines = pfd.read()
    for line in lines.splitlines():
        print(line)
    res = pfd.close()
    # pfd = os.popen('systemctl enable dnf-automatic.timer')
    # lines = pfd.read()
    # for line in lines.splitlines():
    #     print(line)
    # res = res and pfd.close()
    return res

def set_automatic(auto):
    if auto == is_automatic():
        return True
    if auto:
        result = enable_automatic()
    else:
        result = disable_automatic()
    return True if result == None else False

def is_automatic():
    pfd = os.popen('systemctl is-enabled dnf-automatic-install.timer')
    lines = pfd.read()
    for line in lines.splitlines():
        print(line)
    res = pfd.close()
    
    # pfd = os.popen('systemctl is-enabled dnf-automatic.timer')
    # lines = pfd.read()
    # for line in lines.splitlines():
    #     print(line)
    # res = res and pfd.close()
    
    if res:
        return False
    else:
        return True

def formatSize(size):
    if size >= 1024 and size < 1024 * 1024:
        return format(float(size) / (1024), '.1f') + 'k'
    elif size >= 1024 * 1024 and size < 1024 * 1024 * 1024:
        return format(float(size) / (1024 * 1024), '.1f') + 'M'
    elif size >= 1024 * 1024 * 1024 and size < 1024 * 1024 * 1024 * 1024:
        return format(float(size) / (1024 * 1024 * 1024), '.1f') + 'G'
    else:
        return '0'

def update_info():
    updateList = []

    try:
        with dnf.Base() as base:
            cli = dnf.cli.Cli(base)
            cli._read_conf_file()
            try:
                base.init_plugins(cli=cli)
            except:
                pass

            base.pre_configure_plugins()
            base.read_all_repos()
            base.configure_plugins()
            base.fill_sack()
            base.upgrade_all()
            base.resolve()
            output = dnf.cli.output.Output(base, base.conf)
            trans = base.transaction
            if not trans:
                return []

            lst = output.list_transaction(trans)
            for tran in trans:
                if tran.action_name == 'Upgrade' or tran.action_name == 'Erase':
                    pkg = {}
                    pkg['name'] = tran.name
                    pkg['evr'] = tran.evr
                    pkg['repo'] = tran.from_repo  # @System  系统里装的
                    pkg['arch'] = tran.arch
                    pkg['action'] = tran.action_name
                    pkg['size'] = formatSize(tran.pkg._size)
                else:
                    continue
                updateList.append(pkg)
                pass
            return updateList
    except dnf.exceptions.Error as exc:
            return []

if __name__ == "__main__":
    if opts.updateinfo:
        upList = update_info()

        for pkg in upList:
            print(pkg)
    elif opts.installupdates:
        update()

