#!/usr/libexec/platform-python
# -*- coding: UTF-8 -*-
import gi
gi.require_version("Gtk","3.0")
from gi.repository import Gtk as gtk
from gi.repository import GObject as gobject
from gi.repository import GLib as glib
from threading import Thread
import os
import os.path
import sys
import string
import time
import subprocess
import gettext
import dbus
import dbus.mainloop.glib
import logging

t = gettext.translation('nfs-upgrade-tool','/usr/share/locale')
_ = t.gettext

class NfsUpgradeTool(gobject.GObject):
    def __init__(self,parent=None):
        self.have_new_release = False
        self.name = "name"
        self.get_version = "ver"
        self.system_version = "ver1"
        self.builder = gtk.Builder()
        self.builder.add_from_file("/usr/share/nfs-upgrade-tool/data/window.ui")
        self.button_home = self.builder.get_object("button-home-url")
        url = self.button_home.get_uri()
        self.button_home.set_label(url)
        self.window = self.builder.get_object("main-window")
        self.window.connect("destroy",self.window_destroyed)
        self.button_now = self.builder.get_object("button-manual")
        self.button_now.connect("clicked",self.update_now_cb)
        self.button_auto = self.builder.get_object("button-auto")
        self.button_auto.connect("toggled",self.auto_callback)
        self.button_refresh = self.builder.get_object("button-refresh")
        self.button_refresh.connect("clicked",self.refresh_callback)

        self.sys_name=self.builder.get_object("sys-name")
        self.sys_version=self.builder.get_object("sys-version")
        self.current_version=self.builder.get_object("current-version")
        self.new_version=self.builder.get_object("new-version")
        self.notice_label=self.builder.get_object("notice")
        self.notice_box=self.builder.get_object("notice-box")
        self.update_list_view=self.builder.get_object("update-list")
        self.list_store = self.builder.get_object("liststore1")
        self.update_notice = self.builder.get_object("notice-updating")
        self.update_list_spinner = self.builder.get_object("spinner-update-list")

        render = gtk.CellRendererText()
        self.add_list_column(_('Package Name'),render,0)
        self.add_list_column(_('Version'),render,1)
        self.add_list_column(_('Action'),render,2)
        self.add_list_column(_('Size'),render,3)
        self.get_sys_info()
        self.get_current_version()
        self.dbus = dbus.SystemBus()
        try:
            update_bus = self.dbus.get_object('com.nfs.update','/')
            self.dbus_iface = dbus.Interface(update_bus,dbus_interface='com.nfs.update')
            self.dbus_iface.connect_to_signal("Finished",self.dbus_on_finished)
            self.refresh()
        except:
            logging.error("dbus:com.nfs.update is missing.")
            sys.exit(1)
        glib.idle_add(self.do_idle,None)

    def do_idle(self,user_data=None):
        self.check_updating()
        self.check_automatic()
        self.update_new_version()
        self.update_pkgs_list()
        self.window.show_all()
        return False

    def dbus_on_finished(self,action,result,msg=None):
        #print("action = %s, result = %d" % (action, result))
        if action == 'NEW_VERSION':
            self.update_new_version()
        elif action == 'UPDATE_LIST':
            self.update_pkgs_list()
        elif action == 'IS_AUTOMATIC':
            self.check_automatic()
        elif action == 'UPDATE':
            self.check_updating()
            if result:
                self.refresh()
                self.update_notice.set_text(_("Finished"))
            else:
                self.update_notice.set_text(_("Update failed,please try again."))
        elif action == 'IS_UPDATING':
            self.check_updating()

    def check_updating(self):
        dbus_is_updating = self.dbus_iface.get_dbus_method("isUpdating")
        updating = dbus_is_updating()
        self.updating_status_changed(updating)

    def updating_status_changed(self,is_updating=False):
        if is_updating:
            self.button_now.set_sensitive(False)
            self.update_notice.set_text(_("Updating..."))
        else:
            self.button_now.set_sensitive(True)
            self.update_notice.set_text('')
    
    def check_automatic(self):
        dbus_is_automatic = self.dbus_iface.get_dbus_method("isAutomatic")
        is_auto = dbus_is_automatic()
        self.button_auto.set_active(is_auto)

    def update_pkgs_list(self):
        self.list_store.clear()
        self.update_list_spinner.show()
        self.update_list_spinner.start()
        dbus_get_update_pkgs = self.dbus_iface.get_dbus_method("getUpdatePkgs")
        updates = dbus_get_update_pkgs()
        #none = self.builder.get_object("update-none")
        list_count = self.builder.get_object("list-count")
        count = len(updates)
        list_count.show()
        if updates != None and count > 0:
            #none.hide()
            self.button_now.set_sensitive(True)
            for pkg in updates:
                if pkg["name"] == 'nfs-release':
                    self.have_new_release = True
                    self.get_version = pkg["evr"].replace('.nfs4','').replace('.nfs','')
                action = _(pkg["action"])
                self.list_store.append([pkg["name"],pkg["evr"],action,pkg["size"]])
        else:
            self.button_now.set_sensitive(False)
        count_str = _("(%d items)") % count
        list_count.set_text(count_str)
        self.update_list_spinner.stop()
        self.update_list_spinner.hide()
        if self.have_new_release:
            self.update_new_version()

    def add_list_column(self,title,render,index):
        column = gtk.TreeViewColumn(title,render,text=index)
        column.set_min_width(90)
        column.set_resizable(True)
        self.update_list_view.append_column(column)

    def set_notice(self,msg):
        if msg:
            self.notice_box.show()
        else:
            self.notice_box.hide()
        self.notice_label.set_text(msg)

    def get_sys_info(self):
        release_file = '/etc/os-release'	
        lines = open(release_file).readlines()
        for line in lines:
            s = line.split("=",2)
            if len(s) == 2:
                text=s[1].replace('\n','').replace('"','')
                if s[0] == "NAME":
                    self.sys_name.set_text(text)
                if s[0] == "VERSION":
                    self.sys_version.set_text(text)

    def get_current_version(self):
        release_file = '/etc/os-release'
        lines = open(release_file).readlines()
        for line in lines:
            s = line.split("=",2)
            if len(s) == 2:
                text=s[1].replace('\n','').replace('"','')
                if s[0] == "NAME":
                    self.name = text
                if s[0] == "SYSTEM_VERSION":
                    self.system_version = text
                    buf = self.name + " " + self.system_version
                    self.current_version.set_text(buf)

    def update_new_version(self):
        if self.have_new_release:
            new = self.name + " " + self.get_version
        else:
            new = self.name + " " + self.system_version
        self.new_version.set_text(new)


    def refresh(self):
        self.list_store.clear()
        self.set_notice('')
        self.update_notice.set_text('')
        new = _("Fetching...")
        self.new_version.set_text(new)
        self.update_list_spinner.show()
        self.update_list_spinner.start()
        dbus_check_update = self.dbus_iface.get_dbus_method("checkUpdate")
        dbus_check_update()
        self.get_current_version()

    def update_now_cb(self,button):
        dbus_update_new = self.dbus_iface.get_dbus_method("updateNow")
        dbus_update_new()

    def auto_callback(self,button):
        active = button.get_active()
        dbus_set_automatic = self.dbus_iface.get_dbus_method("setAutomatic")
        a = dbus_set_automatic(active)
        if not a:
            #set failed
            button.set_active(not active)
            return
        if active:
            self.set_notice(_("Auto update will run backend in free time."))
        else:
            self.set_notice("")

    def refresh_callback(self,button):
        self.refresh()

    def window_destroyed(self,window):
        gtk.main_quit()

def main():
    NfsUpgradeTool()
    gtk.main()

if __name__ == '__main__':
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    main()
    sys.exit(0)

